@@ -47,36 +47,12 @@ int find_string(int window, char *find, int offset)
 void scrollback_savefile(int window)
 {
 	char *file = NULL;
-	char sel;
 	NEWWIN *mywin = create_popup(8, 40);
 
-	win_header(mywin -> win, "Save buffer to file");
+	win_header(mywin, "Save buffer to file");
 
-	if (lb[window].markset == 'a')
-	{
-		escape_print(mywin -> win, 3, 2, "Write all (^a^) or filtered (^m^)?");
-		mydoupdate(mywin -> win);
-
-		for(;;)
-		{
-			sel = tolower(wait_for_keypress(HELP_SCROLLBACK_SAVEFILE));
-
-			if (sel == 'a' || sel == 'm' || sel == 'q')
-				break;
-
-			wrong_key();
-		}
-	}
-	else
-	{
-		sel = 'a'; /* no need to filter again, it is already filtered */
-	}
-
-	if (sel != 'q')
-	{
-		mvwprintw(mywin -> win, 4, 2, "Select file");
-		file = edit_string(mywin -> win, 5, 40, path_max, 0, NULL, HELP_SCROLLBACK_SAVEFILE_ENTER_FILENAME);
-	}
+	mvwprintw(mywin -> win, 4, 2, "Select file");
+	file = edit_string(mywin, 5, 40, path_max, 0, NULL, HELP_SCROLLBACK_SAVEFILE_ENTER_FILENAME);
 	if (file)
 	{
 		FILE *fh = fopen(file, "w");
@@ -88,19 +64,21 @@ void scrollback_savefile(int window)
 			{
 				if (lb[window].Blines[loop])
 				{
+					char display;
 					char *error;
 					int dummy = -1;
 
 					/* check filter */
-					if (sel == 'a' || check_filter(lb[window].pi[loop], lb[window].Blines[loop], NULL, &error, &dummy) || check_no_suppress_lines_filter(lb[window].pi[loop]) == 0)
-					{
-						fprintf(fh, "%s", lb[window].Blines[loop]);
-					}
-					else if (error)
+					(void)check_filter(lb[window].pi[loop], lb[window].Blines[loop], NULL, &error, &dummy, 0, &display);
+					if (error)
 					{
 						fprintf(fh, "---> Error: %s\n", error);
 						myfree(error);
 					}
+					if (display)
+					{
+						fprintf(fh, "%s", lb[window].Blines[loop]);
+					}
 				}
 			}
 
@@ -109,9 +87,9 @@ void scrollback_savefile(int window)
 		else
 		{
 			mvwprintw(mywin -> win, 6, 2, "Cannot write to file!");
-			mydoupdate(mywin -> win);
+			mydoupdate();
 
-			wait_for_keypress(-1);
+			wait_for_keypress(-1, 0, mywin, 0);
 		}
 	}
 
@@ -126,16 +104,15 @@ int get_lines_needed(char *string, int terminal_width)
 		return 1;
 }
 
-void scrollback_displayline(WINDOW *win, int window, int offset, int terminal_offset)
+void scrollback_displayline(NEWWIN *win, int window, int offset, int terminal_offset)
 {
 	proginfo *cur_line_meta = lb[window].pi[offset];
 	char *cur_line = lb[window].Blines[offset];
 	regmatch_t *pmatch = NULL;
 	int matching_regex = -1;
 	char *error = NULL;
-	char ok;
 
-	wmove(win, terminal_offset, 0);
+	wmove(win -> win, terminal_offset, 0);
 
 	if (cur_line)
 	{
@@ -145,16 +122,17 @@ void scrollback_displayline(WINDOW *win, int window, int offset, int terminal_of
 		}
 		else /* just a buffered line */
 		{
-			ok = check_filter(cur_line_meta, cur_line, &pmatch, &error, &matching_regex);
-			if (ok)
-			{
-				color_print(win, cur_line_meta, cur_line, pmatch, matching_regex);
-			}
-			else if (error)
+			char display;
+			(void)check_filter(cur_line_meta, cur_line, &pmatch, &error, &matching_regex, 0, &display);
+			if (error)
 			{
 				color_print(win, cur_line_meta, error, NULL, -1);
 				myfree(error);
 			}
+			if (display)
+			{
+				color_print(win, cur_line_meta, cur_line, pmatch, matching_regex);
+			}
 		}
 	}
 	else /* an empty line */
@@ -170,11 +148,11 @@ void scrollback(void)
 	int window = 0;
 	NEWWIN *mywin1 = create_popup(7, 22);
 
-	win_header(mywin1 -> win, "Scrollback");
+	win_header(mywin1, "Scrollback");
 
 	if (nfd > 1)
 	{
-		window = select_window(HELP_SCROLLBACK_SELECT_WINDOW);
+		window = select_window(HELP_SCROLLBACK_SELECT_WINDOW, NULL);
 	}
 
 	if (window != -1)
@@ -183,8 +161,8 @@ void scrollback(void)
 		{
 			mvwprintw(mywin1 -> win, 3, 2, "Cannot scrollback:");
 			mvwprintw(mywin1 -> win, 4, 2, "buffering is disabled!");
-			mydoupdate(mywin1 -> win);
-			wait_for_keypress(HELP_SCROLLBACK_NO_MARK);
+			mydoupdate();
+			wait_for_keypress(HELP_SCROLLBACK_NO_MARK, 0, mywin1, 0);
 		}
 	}
 
@@ -204,9 +182,9 @@ void scrollback(void)
 			line_lengths[loop] = get_lines_needed(lb[window].Blines[loop], ncols);
 
 		mywin1 = create_popup(max_y - 4, max_x - 4);
-		wattron(mywin1 -> win, A_REVERSE);
+		inverse_on(mywin1);
 		mvwprintw(mywin1 -> win, max_y - 5, 1, "%02d] %s - %d buffered lines", window, pi[window].filename, lb[window].curpos);
-		wattroff(mywin1 -> win, A_REVERSE);
+		inverse_off(mywin1);
 		wnoutrefresh(mywin1 -> win);
 
 		mywin2 = create_popup(nlines, ncols);
@@ -231,7 +209,7 @@ void scrollback(void)
 					if (lines_used > nlines)
 						break;
 
-					scrollback_displayline(mywin2 -> win, window, offset + index, prev_lines_used);
+					scrollback_displayline(mywin2, window, offset + index, prev_lines_used);
 					index++;
 				}
 
@@ -240,14 +218,14 @@ void scrollback(void)
 
 			if (redraw == 1)
 			{
-				mydoupdate(mywin2 -> win);
+				mydoupdate();
 
 				redraw = 0;
 			}
 
-			c = wait_for_keypress(HELP_SCROLLBACK_HELP);
+			c = wait_for_keypress(HELP_SCROLLBACK_HELP, 0, NULL, 0);
 
-			if (toupper(c) == 'Q' || toupper(c) == 'X')
+			if (toupper(c) == 'Q' || toupper(c) == 'X' || c == 7)
 			{
 				break;
 			}
@@ -261,7 +239,7 @@ void scrollback(void)
 				n_to_move = line_lengths[offset];
 				winsdelln(mywin2 -> win, n_to_move);
 
-				scrollback_displayline(mywin2 -> win, window, offset, 0);
+				scrollback_displayline(mywin2, window, offset, 0);
 
 				redraw = 1;
 			}
@@ -312,9 +290,9 @@ void scrollback(void)
 				int new_f_index;
 				NEWWIN *mywin = create_popup(5, 40);
 
-				win_header(mywin -> win, "Find");
+				win_header(mywin, "Find");
 
-				dummy = edit_string(mywin -> win, 3, 40, 80, 0, find, HELP_SCROLLBACK_EDIT_SEARCH_STRING);
+				dummy = edit_string(mywin, 3, 40, 80, 0, find, HELP_SCROLLBACK_EDIT_SEARCH_STRING);
 				myfree(find);
 				find = dummy;
 
@@ -374,11 +352,11 @@ void delete_mark(void)
 	int window = 0;
 	NEWWIN *mywin = create_popup(13, 40);
 
-	win_header(mywin -> win, "Delete mark");
+	win_header(mywin, "Delete mark");
 
 	if (nfd > 1)
 	{
-		window = select_window(HELP_SCROLLBACK_DELETE_MARK_SELECT_WINDOW);
+		window = select_window(HELP_SCROLLBACK_DELETE_MARK_SELECT_WINDOW, NULL);
 	}
 
 	if (window != -1)
@@ -386,8 +364,8 @@ void delete_mark(void)
 		if (lb[window].markset == 0)
 		{
 			mvwprintw(mywin -> win, 3, 2, "No mark was set!");
-			mydoupdate(mywin -> win);
-			wait_for_keypress(HELP_SCROLLBACK_DELETE_MARK_NO_MARK);
+			mydoupdate();
+			wait_for_keypress(HELP_SCROLLBACK_DELETE_MARK_NO_MARK, 0, mywin, 0);
 		}
 		else
 		{
